@@ -79,6 +79,18 @@ gboolean gdm_is_a_no_password_user (const char *user);
 
 int gdm_open_dev_null (mode_t mode);
 
+void gdm_unset_signals (void);
+
+char * gdm_locale_to_utf8 (const char *text);
+char * gdm_locale_from_utf8 (const char *text);
+
+void gdm_saveenv (void);
+/* leaks */
+void gdm_restoreenv (void);
+
+void gdm_sleep_no_signal (int secs);
+void gdm_reset_limits (void);
+
 #endif /* GDM_MISC_H */
 
 /* EOF */
