@@ -86,14 +86,14 @@ static void gdm_chooser_warn (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
 /* Exported for glade */
 gboolean gdm_chooser_cancel (void);
-gboolean gdm_chooser_manage (GtkButton *button, gpointer data);
-gboolean gdm_chooser_browser_select (GtkWidget *widget,
-				     gint selected,
-				     GdkEvent *event);
-gboolean gdm_chooser_browser_unselect (GtkWidget *widget,
-				       gint selected,
-				       GdkEvent *event);
-gboolean gdm_chooser_xdmcp_discover (void);
+void gdm_chooser_manage (GtkButton *button, gpointer data);
+void gdm_chooser_browser_select (GtkWidget *widget,
+				 gint selected,
+				 GdkEvent *event);
+void gdm_chooser_browser_unselect (GtkWidget *widget,
+				   gint selected,
+				   GdkEvent *event);
+void gdm_chooser_xdmcp_discover (void);
 void display_chooser_information (void);
 
 static guint scan_time_handler = 0;
@@ -152,6 +152,14 @@ static GdkPixbuf *defhostimg;
 static GtkWidget *browser;
 static GdmChooserHost *curhost;
 
+static void
+setup_cursor (GdkCursorType type)
+{
+	GdkCursor *cursor = gdk_cursor_new (type);
+	gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
+	gdk_cursor_unref (cursor);
+}
+
 static void
 gdm_chooser_host_dispose (GdmChooserHost *host)
 {
@@ -159,7 +167,7 @@ gdm_chooser_host_dispose (GdmChooserHost *host)
 	return;
 
     if (host->picture != NULL)
-	    gdk_pixbuf_unref (host->picture);
+	    g_object_unref (G_OBJECT (host->picture));
     host->picture = NULL;
 
     g_free (host->name);
@@ -216,6 +224,10 @@ gdm_chooser_host_alloc (const char *hostname,
 	    return host;
 
     hostimg = g_strconcat (GdmHostIconDir, "/", hostname, NULL);
+    if (access (hostimg, R_OK) != 0) {
+	    g_free (hostimg);
+	    hostimg = g_strconcat (GdmHostIconDir, "/", hostname, ".png", NULL);
+    }
 
     if (access (hostimg, R_OK) == 0 &&
 	(img = gdk_pixbuf_new_from_file (hostimg, NULL)) != NULL) {
@@ -238,11 +250,11 @@ gdm_chooser_host_alloc (const char *hostname,
 		host->picture = gdk_pixbuf_scale_simple (img, w, h,
 							 GDK_INTERP_BILINEAR);
 	else
-		host->picture = gdk_pixbuf_ref (img);
+		host->picture = g_object_ref (G_OBJECT (img));
 	
-	gdk_pixbuf_unref (img);
+	g_object_unref (G_OBJECT (img));
     } else if (defhostimg != NULL) {
-	    host->picture = gdk_pixbuf_ref (defhostimg);
+	    host->picture = (GdkPixbuf *)g_object_ref (G_OBJECT (defhostimg));
     }
 
     g_free (hostimg);
@@ -510,7 +522,7 @@ ping_try (gpointer data)
 		return TRUE;
 }
 
-gboolean 
+void
 gdm_chooser_xdmcp_discover (void)
 {
     GList *hl = hosts;
@@ -542,8 +554,6 @@ gdm_chooser_xdmcp_discover (void)
     if (ping_try_handler > 0)
 	    g_source_remove (ping_try_handler);
     ping_try_handler = g_timeout_add (PING_TIMEOUT, ping_try, NULL);
-
-    return TRUE;
 }
 
 #ifndef ishexdigit
@@ -640,6 +650,8 @@ gdm_chooser_xdmcp_init (char **hosts)
     gdm_chooser_add_hosts (hosts);
 
     channel = g_io_channel_unix_new (sockfd);
+    g_io_channel_set_encoding (channel, NULL, NULL);
+    g_io_channel_set_buffered (channel, FALSE);
     g_io_add_watch_full (channel, G_PRIORITY_DEFAULT,
 			G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
 			gdm_chooser_decode_packet,
@@ -715,7 +727,7 @@ gdm_chooser_cancel (void)
 }
 
 
-gboolean
+void
 gdm_chooser_manage (GtkButton *button, gpointer data)
 {
     if (scan_time_handler > 0) {
@@ -728,8 +740,6 @@ gdm_chooser_manage (GtkButton *button, gpointer data)
    
     closelog();
     gtk_main_quit();
-   
-    return TRUE;
 }
 
 
@@ -828,52 +838,24 @@ gdm_nth_willing_host (int n)
 }
 
 
-gboolean 
+void
 gdm_chooser_browser_select (GtkWidget *widget, gint selected, GdkEvent *event)
 {
-    if (!widget || !event)
-	return TRUE;
-
-    switch (event->type) {
-	
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
 	curhost = gdm_nth_willing_host (selected);
 	gtk_widget_set_sensitive (manage, TRUE);
-	break;
 
-    case GDK_2BUTTON_PRESS:
-	curhost = gdm_nth_willing_host (selected);
-	gdm_chooser_manage (NULL, NULL);
-	break;
-	
-    default: 
-	break;
-    }
-    
-    return TRUE;
+	if (event != NULL &&
+	    event->type == GDK_2BUTTON_PRESS) {
+		gdm_chooser_manage (NULL, NULL);
+	}
 }
 
 
-gboolean
+void
 gdm_chooser_browser_unselect (GtkWidget *widget, gint selected, GdkEvent *event)
 {
-    if (!event) 
-	return TRUE;
-
-    switch (event->type) {
-	
-    case GDK_BUTTON_PRESS:
-    case GDK_BUTTON_RELEASE:
 	curhost = NULL;
 	gtk_widget_set_sensitive (manage, FALSE);
-	break;
-	
-    default:
-	break;
-    }
-
-    return TRUE;
 }
 
 void
@@ -906,12 +888,8 @@ static void
 gdm_chooser_gui_init (void)
 {
 	glade_helper_add_glade_directory (GDM_GLADE_DIR);
+	glade_helper_search_gnome_dirs (FALSE);
 
-#if 0
-    GtkStyle  *style;
-    GdkColor  bbg = { 0, 0xFFFF, 0xFFFF, 0xFFFF };
-#endif
-   
     /* Enable theme */
     if (GdmGtkRC)
 	gtk_rc_parse (GdmGtkRC);
@@ -1017,14 +995,18 @@ set_background (void)
 			gdk_color_parse ("#007777", &color);
 		}
 
-		colormap = gdk_window_get_colormap (GDK_ROOT_PARENT ());
+		colormap = gdk_drawable_get_colormap
+			(gdk_get_default_root_window ());
 		/* paranoia */
 		if (colormap != NULL) {
+			gboolean success;
 			gdk_error_trap_push ();
 
-			gdk_color_alloc (colormap, &color);
-			gdk_window_set_background (GDK_ROOT_PARENT (), &color);
-			gdk_window_clear (GDK_ROOT_PARENT ());
+			gdk_colormap_alloc_colors (colormap, &color, 1,
+						   FALSE, TRUE, &success);
+
+			gdk_window_set_background (gdk_get_default_root_window (), &color);
+			gdk_window_clear (gdk_get_default_root_window ());
 
 			gdk_flush ();
 			gdk_error_trap_pop ();
@@ -1093,6 +1075,9 @@ main (int argc, char *argv[])
 	    exit (1);
     }
 
+    /* Should be a watch already, but just in case */
+    setup_cursor (GDK_WATCH);
+
     glade_init();
 
     gdm_chooser_parse_config();
@@ -1122,6 +1107,8 @@ main (int argc, char *argv[])
 	    gtk_widget_show_all (dialog);
 	    gdm_wm_center_window (GTK_WINDOW (dialog));
 
+	    setup_cursor (GDK_LEFT_PTR);
+
 	    gtk_dialog_run (GTK_DIALOG (dialog));
 
 	    return EXIT_SUCCESS;
@@ -1145,7 +1132,7 @@ main (int argc, char *argv[])
     gdm_chooser_xdmcp_init (hosts);
     poptFreeContext (ctx);
 
-    if (g_getenv ("RUNNING_UNDER_GDM") != NULL) {
+    if (RUNNING_UNDER_GDM) {
 	    guint sid = g_signal_lookup ("event",
 					 GTK_TYPE_WIDGET);
 	    g_signal_add_emission_hook (sid,
@@ -1170,6 +1157,8 @@ main (int argc, char *argv[])
 	    gdm_wm_focus_window (GDK_WINDOW_XWINDOW (chooser->window));
     }
 
+    setup_cursor (GDK_LEFT_PTR);
+
     gtk_main();
 
     exit (EXIT_SUCCESS);
