@@ -5,6 +5,7 @@
 #include <gdk/gdkkeysyms.h>
 #include "greeter_item_pam.h"
 #include "greeter_parser.h"
+#include "greeter.h"
 #include "gdm.h"
 #include "gdmwm.h"
 #include "vicious.h"
@@ -20,6 +21,8 @@ extern gboolean GdmSystemMenu;
 
 gchar *greeter_current_user = NULL;
 
+gboolean require_quarter = FALSE;
+
 void
 greeter_item_pam_set_user (const char *user)
 {
@@ -27,11 +30,44 @@ greeter_item_pam_set_user (const char *user)
   greeter_current_user = g_strdup (user);
 }
 
+static gboolean
+evil (GtkEntry *entry, const char *user)
+{
+	/* do not translate */
+	if (strcmp (user, "Gimme Random Cursor") == 0) {
+		greeter_setup_cursor (((rand () >> 3) % (GDK_LAST_CURSOR/2)) * 2);
+		gtk_entry_set_text (GTK_ENTRY (entry), "");
+		return TRUE;
+		/* do not translate */
+	} else if (strcmp (user, "Require Quater") == 0 ||
+		   strcmp (user, "Require Quarter") == 0) {
+		/* btw, note that I misspelled quarter before and
+		 * thus this checks for Quater as well as Quarter to
+		 * keep compatibility which is obviously important for
+		 * something like this */
+		require_quarter = TRUE;
+		gtk_entry_set_text (GTK_ENTRY (entry), "");
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
 {
   char *tmp;
+  const char *str;
   GreeterItemInfo *error_info;
+
+  str = gtk_entry_get_text (GTK_ENTRY (entry));
+  if (entry_is_login &&
+      /* evilness */
+      evil (entry, str))
+    {
+      gtk_entry_set_text (GTK_ENTRY (entry), "");
+      return;
+    }
   
   gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
 
